/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getTestElement } from '../../support/cypress-helper';
import { convertToDataTestId } from '../../support/tech.util';
import { AttachmentContainerE2EComponent } from '../attachment/attachment.e2e.component';
import { UserProfileE2EComponent } from '../user-profile/user-profile.component.e2e';

export class PostfachMailE2EComponent {
  private readonly createMailButtonWithText: string = 'create-mail-button-with-text';
  private readonly createMailButtonWithoutText: string = 'create-mail-button-without-text';
  private readonly list: string = 'postfach-mail-list';
  private readonly noPostfachText: string = 'no-postfach-text';
  private readonly attachments: string = 'postfach-nachricht-attachments-container';
  private readonly downloadButtonWithIcon: string = 'postfach-pdf-export-button';
  private readonly downloadButtonWithLabel: string = 'postfach-pdf-export-label-button';

  private readonly locatorRoot: string = 'postfach-nachrichten-container-in-vorgang';

  public getRoot(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.locatorRoot);
  }

  public getCreateButtonWithText(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.createMailButtonWithText);
  }

  public getCreateButtonWithoutText(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.createMailButtonWithoutText);
  }

  public getList(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.list);
  }

  public getNoPostfachText(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.noPostfachText);
  }

  public getListItem(subject: string): PostfachMailListItemE2EComponent {
    return new PostfachMailListItemE2EComponent(subject);
  }

  public getAttachments(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.attachments);
  }

  public getDownloadButtonWithIcon(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.downloadButtonWithIcon);
  }

  public getDownloadButtonWithLabel(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.downloadButtonWithLabel);
  }
}

export class PostfachMailListItemE2EComponent {
  private readonly createdAt: string = 'mail-created-at';
  private readonly subject: string = 'mail-subject';
  private readonly text: string = 'mail-text';
  private readonly replyIcon: string = 'reply-icon';
  private readonly sentAt: string = 'mail-sent-at';
  private readonly editButton: string = 'postfach-nachricht-edit-button-container';

  private readonly sendErrorText: string = 'mail-send-error-text';
  private readonly sendErrorIcon: string = 'mail-send-error-icon';
  private readonly resendButton: string = 'resend-nachricht-button';

  private readonly attachmentContainer: AttachmentContainerE2EComponent = new AttachmentContainerE2EComponent();

  private root: string;

  constructor(subject: string) {
    this.root = convertToDataTestId(subject) + '-item';
  }

  public getRoot(): Cypress.Chainable<JQuery<Element>> {
    return getTestElement(this.root);
  }

  public getEditButton(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.editButton);
  }

  public getSubject(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.subject);
  }

  public getText(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.text);
  }

  public getCreatedAt(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.createdAt);
  }

  public getUserProfile(): UserProfileE2EComponent {
    return new UserProfileE2EComponent(this.root);
  }

  public getReplyIcon(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.replyIcon);
  }

  public getSentAt(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.sentAt);
  }

  public getSendErrorText(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.sendErrorText);
  }

  public getSendErrorIcon(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.sendErrorIcon);
  }

  public getResendButton(): Cypress.Chainable<JQuery<Element>> {
    return this.getRoot().findTestElementWithClass(this.resendButton);
  }

  public getAttachmentContainer(): AttachmentContainerE2EComponent {
    return this.attachmentContainer;
  }
}
