import { PostfachMailE2EComponent } from 'apps/alfa-e2e/src/components/postfach/postfach-mail.e2e.component';
import { CommandE2E, CommandOrderE2E, CommandStatusE2E } from 'apps/alfa-e2e/src/model/command';
import { createCommand, initCommands } from 'apps/alfa-e2e/src/support/command-util';
import { APP_HELPER, VORGANG_NAVIGATOR } from 'apps/alfa-e2e/src/support/e2e';
import {
  createPostfachNachrichtAttachedItem,
  createPostfachNachrichtReplyItem,
  createSendPostfachNachrichtItem,
} from 'apps/alfa-e2e/src/support/postfach-nachricht.util';
import { getUserSabineId } from 'apps/alfa-e2e/src/support/user-util';
import { initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import { VorgangE2E } from '../../../model/vorgang';
import {
  DirectionE2E,
  PostfachMailItemE2E,
  PostfachNachrichtMessageCodeE2E,
  VorgangAttachedItemE2E,
} from '../../../model/vorgang-attached-item';
import { containsSpinner } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { exist } from '../../../support/cypress.util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe('Postfach Nachricht pending', () => {
  const vorgangPage: VorgangPage = new VorgangPage();
  const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();

  //SEND
  const vorgangWithPendingSendCommand: VorgangE2E = { ...createVorgang(), name: 'VorgangWithPendingSendCommand' };
  const postfachSendNachrichtItem: PostfachMailItemE2E = {
    ...createSendPostfachNachrichtItem(),
    sentSuccessful: false,
  };
  const postfachNachrichtAttachedItem: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[1], vorgangWithPendingSendCommand._id.$oid),
    item: postfachSendNachrichtItem,
  };
  const pendingSendCommand: CommandE2E = {
    ...createCommand(),
    order: CommandOrderE2E.SEND_POSTFACH_NACHRICHT,
    status: CommandStatusE2E.PENDING,
    finishedAt: null,
    body: postfachSendNachrichtItem,
  };

  //RESEND 1
  const vorgangWithResendCommand: VorgangE2E = buildVorgang(objectIds[1], 'VorgangWithResendCommand');
  const resendPostfachNachricht1: PostfachMailItemE2E = {
    ...createPostfachNachrichtReplyItem(),
    subject: 'Failed Outgoing Postfach Nachricht 1',
    createdBy: getUserSabineId(),
    direction: DirectionE2E.OUT,
    sentAt: '2022-12-02T15:00:00.790Z[UTC]',
    sentSuccessful: false,
    messageCode: PostfachNachrichtMessageCodeE2E.PROCESSING_FAILED,
  };
  const resendPostfachNachrichtVorgangAttachedItem1: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[2], vorgangWithResendCommand._id.$oid),
    item: resendPostfachNachricht1,
  };
  const pendingResendCommand1: CommandE2E = {
    ...createCommand(),
    _id: { $oid: objectIds[1] },
    vorgangId: vorgangWithResendCommand._id.$oid,
    order: CommandOrderE2E.RESEND_POSTFACH_NACHRICHT,
    status: CommandStatusE2E.PENDING,
    finishedAt: null,
    body: resendPostfachNachricht1,
    relationId: resendPostfachNachrichtVorgangAttachedItem1._id.$oid,
  };

  //RESEND 2
  const resendPostfachNachricht2: PostfachMailItemE2E = {
    ...resendPostfachNachricht1,
    subject: 'Failed Outgoing Postfach Nachricht 2',
  };
  const resendPostfachNachrichtVorgangAttachedItem2: VorgangAttachedItemE2E = {
    ...createPostfachNachrichtAttachedItem(objectIds[3], vorgangWithResendCommand._id.$oid),
    item: resendPostfachNachricht2,
  };
  const pendingResendCommand2: CommandE2E = {
    ...pendingResendCommand1,
    _id: { $oid: objectIds[2] },
    body: resendPostfachNachricht2,
    relationId: resendPostfachNachrichtVorgangAttachedItem2._id.$oid,
  };

  before(() => {
    initVorgaenge([vorgangWithPendingSendCommand, vorgangWithResendCommand]);
    initVorgangAttachedItem([
      postfachNachrichtAttachedItem,
      resendPostfachNachrichtVorgangAttachedItem1,
      resendPostfachNachrichtVorgangAttachedItem2,
    ]);
    initCommands([pendingSendCommand, pendingResendCommand1, pendingResendCommand2]);

    APP_HELPER.loginAsSabine();
  });

  describe('send command on area in vorgang detail', () => {
    it('should have item in list', () => {
      VORGANG_NAVIGATOR.openVorgang(vorgangWithPendingSendCommand.name);

      exist(postfachMailContainer.getListItem(postfachSendNachrichtItem.subject).getRoot());
    });

    it('should show spinner on buttons', () => {
      containsSpinner(postfachMailContainer.getCreateButtonWithText());
      containsSpinner(postfachMailContainer.getCreateButtonWithoutText());
    });

    it('should show edit button', () => {
      exist(postfachMailContainer.getListItem(postfachSendNachrichtItem.subject).getEditButton());
    });
  });

  describe('resend commands area in vorgang detail', () => {
    it('should have items in list', () => {
      VORGANG_NAVIGATOR.openVorgang(vorgangWithResendCommand.name);

      exist(postfachMailContainer.getListItem(resendPostfachNachricht1.subject).getRoot());
      exist(postfachMailContainer.getListItem(resendPostfachNachricht2.subject).getRoot());
    });

    it('should show spinner on buttons', () => {
      containsSpinner(postfachMailContainer.getListItem(resendPostfachNachricht1.subject).getResendButton());
      containsSpinner(postfachMailContainer.getListItem(resendPostfachNachricht2.subject).getResendButton());
    });
  });
});
