/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { faker } from '@faker-js/faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { CommandLinkRel, CommandListLinkRel } from '../src/lib/command.linkrel';
import { CommandErrorMessage } from '../src/lib/command.message';
import {
  Command,
  CommandListResource,
  CommandOrder,
  CommandResource,
  CommandStatus,
  CreateCommand,
  CreateCommandProps,
  CreateCommandPropsWithoutResource,
  PendingCommandMap,
} from '../src/lib/command.model';

export function createCommand(): Command {
  return {
    body: null,
    createdAt: faker.date.past(),
    finishedAt: faker.date.past(),
    order: faker.word.sample(),
    status: CommandStatus.FINISHED,
    errorMessage: null,
  };
}

export function createCommandResource(linkRel: string[] = []): CommandResource {
  return toResource(createCommand(), linkRel);
}

export function createCommandResources(linkRelations: string[] = []): CommandResource[] {
  return times(10, () => toResource(createCommandResource(), [...linkRelations]));
}

export function createCommandStateResource(linkRel: string[] = []): StateResource<CommandResource> {
  return createStateResource(createCommandResource(linkRel));
}

export function createCommandListResource(
  commandResources: CommandResource[] = createCommandResources(),
  linkRelations: string[] = [],
): CommandListResource {
  return toResource({}, [...linkRelations], {
    [CommandListLinkRel.COMMAND_LIST]: commandResources,
  });
}

export function createCommandErrorResource(linkRelations: string[] = []): CommandResource {
  return {
    ...createCommandResource(linkRelations),
    errorMessage: CommandErrorMessage.CONCURRENT_MODIFICATION,
  };
}

export function createCommandErrorStateResource(linkRelations: string[] = []): StateResource<CommandResource> {
  return createStateResource(createCommandErrorResource(linkRelations));
}

export function createCreateCommand(order: CommandOrder = CommandOrder.VORGANG_ANNEHMEN): CreateCommand {
  return { order, body: null };
}

export function createCreateCommandProps(): CreateCommandProps {
  return { ...createCreateCommandPropsWithoutResource(), resource: createCommandResource() };
}

export function createCreateCommandPropsWithoutResource(): CreateCommandPropsWithoutResource {
  return {
    linkRel: faker.internet.url(),
    command: createCreateCommand(),
    snackBarMessage: faker.word.sample(5),
  };
}

export function createSuccessfullyDoneCommandStateResource(): StateResource<CommandResource> {
  return createCommandStateResource([CommandLinkRel.EFFECTED_RESOURCE]);
}

export function createSuccessfullyDoneCommandResource(): CommandResource {
  return createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
}

export function createPendingCommandMap(): PendingCommandMap {
  return { [faker.internet.url()]: createCommandStateResource() };
}
